/*
 *	KhepComWin.c
 *
 *	Khepera remote control via serial link on Windows
 *
 *	(c) 1995-2000, Yves Piguet <yves.piguet@calerga.com>
 *
 *	Includes code by Patrick Saucy
 */

#include "KhepCom.h"

#ifndef _WIN32
#define _WIN32
#endif

#include <windows.h>
//#include <winbase.h>

#define KHEPERA_CREATEFILE_E		41
#define KHEPERA_GETCOMMSTATE_E		42
#define KHEPERA_SETCOMMSTATE_E		43
#define KHEPERA_WRITEFILE_E			44
#define KHEPERA_READFILE_E			45
#define KHEPERA_PARAM_E				46
#define KHEPERA_SET_TIMEOUT_E		47
#define KHEPERA_TIMEOUT_E			49

#define BYTE_SIZE_K					8

static int OpenSerial(HANDLE *hCom, char *port, DWORD baudrate, long tmeout)
{
	DCB dcb;
	COMMTIMEOUTS timeout;
	
	// open serial port
	
	*hCom = CreateFile(port, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if(*hCom == INVALID_HANDLE_VALUE)
		return KHEPERA_CREATEFILE_E;
	
	// set parameters
	
	if(!GetCommState(*hCom, &dcb))				// get default values
		return KHEPERA_GETCOMMSTATE_E;
	dcb.BaudRate = baudrate;
	dcb.ByteSize = BYTE_SIZE_K;
	dcb.Parity = NOPARITY;
	dcb.StopBits = TWOSTOPBITS;
	if (!SetCommState(*hCom, &dcb))
		return KHEPERA_SETCOMMSTATE_E;
	
	// set timeout
	
	timeout.ReadIntervalTimeout = tmeout;
	timeout.ReadTotalTimeoutMultiplier = 0;
	timeout.ReadTotalTimeoutConstant = tmeout;
	timeout.WriteTotalTimeoutMultiplier = 0;
	timeout.WriteTotalTimeoutConstant = tmeout;
	if(!SetCommTimeouts(*hCom,&timeout))
		return KHEPERA_SET_TIMEOUT_E;

	return 0;
}

static int GetAnswer(HANDLE hCom,
					int stopAtCr,
					char **answer, long answerMaxLength, long *length,
					reallocPtr myRealloc)
{
	char car;
	DWORD bytes_read;
	char *oldPtr;
	
	if( (hCom==INVALID_HANDLE_VALUE) || (answer==NULL) )
		return KHEPERA_PARAM_E;
	*length = 0;
	for (;;)
	{
		if (!ReadFile(hCom,&car,1,&bytes_read,NULL))
			return KHEPERA_READFILE_E;
		if (bytes_read == 0)

		{
			if (*length < answerMaxLength)

				(*answer)[*length] = '\0';

			return KHEPERA_TIMEOUT_E;
		}


		if (*length + 2>= answerMaxLength)
		{
			oldPtr = *answer;
			*answer = (char *)myRealloc(oldPtr,
							answerMaxLength < 256 ? 512 : 2 * answerMaxLength);
			
			if (*answer)	// ok
				answerMaxLength = answerMaxLength < 256 ? 512 : 2 * answerMaxLength;
			else
			{
				*answer = oldPtr;
				break;
			}
		}
		
		if (stopAtCr && car == '\n')
		{
			(*answer)[*length] = '\0';
			break;
		}
		else if (car != '\r')
			(*answer)[(*length)++] = car;
	}
	
	if (*length < answerMaxLength)

		(*answer)[*length] = '\0';
		
	return 0;
}

int KOpenSerial(long port, long brate, long timeout, KPortRef *portRef)
	// open and configure the serial port for communicating with Khepera
	// same parameters as SendCommandToKhepera; return a port reference in *portRef
	//  and an error code
{
	int err;
	char *portName;
	HANDLE hCom;
	char car;
	DWORD bytes_read;
	
	if (port < 0 || port >= 4)
		return kKhepComBadPortErr;
	portName = port == 0 ? "COM1" : port == 1 ? "COM2" : port == 2 ? "COM3" : "COM4";
	
	if (brate != 150 && brate != 300 && brate != 600 && brate != 1200 &&
			brate != 1800 && brate != 2400 && brate != 4800 && brate != 7200 &&
			brate != 9600 && brate != 14400 && brate != 19200 && brate != 28800 &&
			brate != 38400 && brate != 57600)
		return kKhepComBadBaudRateErr;
	
	// flush input with a small timeout
	err = OpenSerial(&hCom, portName, brate, 10);
	while (ReadFile(hCom, &car, 1, &bytes_read, NULL) && bytes_read == 1)
		;
	CloseHandle(hCom);
	
	// reopen the serial port with the specified timeout
	err = OpenSerial(&hCom, portName, brate, timeout);
	if (err != 0)
		return kKhepDeviceErr;
	portRef->priv[0] = (long)hCom;
	return kKhepNoErr;
}

void KCloseSerial(KPortRef *portRef)
	// close the serial port
{
	HANDLE hCom;
	
	hCom = (HANDLE)portRef->priv[0];
	CloseHandle(hCom);
}

int KGetSerialDeviceName(long i, char *name, long *nameLength)
	// retrieve the name of serial device i (i >= 0)
	// ret. 1 if ok, 0 if none
{
	if (i < 0 || i > 7)
		return 0;
	name[0] = 'C';
	name[1] = 'O';
	name[2] = 'M';
	name[3] = '1' + i;
	name[4] = '\0';
	if (nameLength)
		*nameLength = 4;
	
	return 1;
}

void KFlush(KPortRef *portRef)
{
	HANDLE hCom;
	char car;
	DWORD bytes_read;
	
	hCom = (HANDLE)portRef->priv[0];
	
	while (ReadFile(hCom, &car, 1, &bytes_read, NULL) && bytes_read == 1)
		;
}

int KSend(KPortRef *portRef, const char *buf, long length)
{
	HANDLE hCom;
	DWORD bytes_written;
	
	hCom = (HANDLE)portRef->priv[0];
	
	if (!WriteFile(hCom, buf, (DWORD)length, &bytes_written, NULL))
		return kKhepComErr;
	
	return kKhepNoErr;
}

int KReceive(KPortRef *portRef, char *buf, long maxLength, long *length)
{
	HANDLE hCom;
	char car;
	DWORD bytes_read;
	
	hCom = (HANDLE)portRef->priv[0];
	for (*length = 0; *length < maxLength; (*length)++)
	{
		if (!ReadFile(hCom, &car, 1, &bytes_read, 0))
			return kKhepComErr;
		if (bytes_read == 0)
		{
			if (*length < maxLength)
				buf[*length] = '\0';
			return kKhepComTimeoutErr;
		}
		buf[*length] = car;
	}
	
	return kKhepNoErr;
}

int KSendCommand(KPortRef *portRef,	// as returned by KOpenSerial
						char *command, long cmdLength,
						char **answer, long answerMaxLength, long *answerLength,
						int stopAtCr,	// if false, read until timeout
						reallocPtr myRealloc)
	// use myRealloc if answerMaxLength is not large enough
	// answer is null-terminated (C-string)
{
	int err;
	HANDLE hCom;
	DWORD bytes_written;
	
	hCom = (HANDLE)portRef->priv[0];
	
	if (!WriteFile(hCom,command, (DWORD)cmdLength, &bytes_written, NULL))
		return kKhepComErr;
	
	err = GetAnswer(hCom, stopAtCr, answer, answerMaxLength, answerLength, myRealloc);
	
	if (stopAtCr && err == KHEPERA_TIMEOUT_E)
		return kKhepComTimeoutErr;
	if (err == KHEPERA_READFILE_E)
		return kKhepComErr;
	return kKhepNoErr;
}


int KOpenSendClose(long port, long brate, long timeout,
						char *command, long cmdLength,
						char **answer, long answerMaxLength, long *answerLength,
						int stopAtCr,	// if false, read until timeout
						reallocPtr myRealloc)
	// perform KOpenSerial, KSendCommand, and KCloseSerial
{
	KPortRef p;
	int err;
	
	err = KOpenSerial(port, brate, timeout, &p);
	if (err != kKhepNoErr)
		return kKhepNoErr;
	err = KSendCommand(&p, command, cmdLength,
						answer, answerMaxLength, answerLength,
						stopAtCr, myRealloc);
	KCloseSerial(&p);
	return err;
}
