/*
	khepcmd		Send a command to Khepera
	
	The calling syntax is:
	
		result = khepcmd(cmd)
	
	or
	
		result = khepcmd(cmd, [port,baudrate,timeout])
	
	Arguments:
	
	cmd:	the command to send (without the carriage return) as a string
	port:	0 for the modem port, 1 for the printer port (default = 0)
	result:	the reply of Khepera



	Under Windows, compile by typing in Matlab's command line



		mex -output khepcmd KhepcmdMEX.c KhepComWin.c
	
	Copyright 1998, Yves Piguet
	All Righs Reserved
*/

//#include "CodeWarrior.MEX.prefix"

#include "mex.h"

#include "KhepCom.h"

/* Input Arguments */

#define	CMD_IN		prhs[0]
#define	CONFIG_IN	prhs[1]

/* Output Arguments */

#define	RESULT_OUT	plhs[0]

// malloc or realloc, sent to SendCommandToKhepera()

static void *myRealloc(void *ptr, long size)
{
	if (ptr)
		return mxRealloc(ptr, (size_t)size);
	else
		return mxMalloc((size_t)size);
}

// Main Function

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	char string[512], *answer;
	long cmdLength, answerLength;
	short port;
	long rate;
	double timeout;
	int stopAtCr;
	int err;
	
	/* Checks for proper number of arguments */
	
	if (nrhs < 1 || nrhs > 2 || nlhs > 1)
		mexErrMsgTxt("KHEPCMD syntax: reply = khepcmd(cmd,config)");
	
	/* Gets the parameters */
	
	if (!mxIsChar(CMD_IN) || mxIsSparse(CMD_IN) || mxGetM(CMD_IN) != 1)
		mexErrMsgTxt("KHEPCMD first arg. must be a string");
	if (mxGetN(CMD_IN) > 510)
		mexErrMsgTxt("KHEPCMD first arg. is limited to 510 characters");
	
	cmdLength = mxGetN(CMD_IN);
	mxGetString(CMD_IN, string, cmdLength + 1);
	
	if (nrhs >= 2)
	{
		if(!mxIsDouble(CONFIG_IN) || mxIsComplex(CONFIG_IN) ||
									mxGetM(CONFIG_IN) != 1 || mxGetN(CONFIG_IN) > 4) 
			mexErrMsgTxt("KHEPCMD second arg. must be a row vector of up to four numbers");
		port = (short)mxGetPr(CONFIG_IN)[0];
		if (port != 0 && port != 1)
			mexErrMsgTxt("KHEPCMD port must be 0 (modem) or 1 (printer)");
		if (mxGetN(CONFIG_IN) >= 2)
			rate = (long)mxGetPr(CONFIG_IN)[1];
		else
			rate = 19200;
		if (mxGetN(CONFIG_IN) >= 3)
			timeout = mxGetPr(CONFIG_IN)[2];
		else
			timeout = 1;
		if (mxGetN(CONFIG_IN) >= 4)
			stopAtCr = mxGetPr(CONFIG_IN)[3] != 0;
		else
			stopAtCr = 1;
	}
	else
	{
		port = 0;
		rate = 19200;
		timeout = 1;
	}
	
#ifdef dynamic_reply
	answer = 0;
	err = KOpenSendClose(port, rate,
					timeout < 0 ? 60000L : timeout > 60 ? 60000L : (long)(timeout * 1000),
					string, cmdLength,
					&answer, 0, &answerLength, stopAtCr,
					myRealloc);
	
	if (err == kKhepNoErr && nlhs >= 1)
		RESULT_OUT = mxCreateString(answer ? answer : "");
	
	if (answer)
		mxFree(answer);
#else
	{
		char reply[2048];
		answer = reply;
		err = KOpenSendClose(port, rate,
						timeout < 0 ? 60000L : timeout > 60 ? 60000L : (long)(timeout * 1000),
						string, cmdLength,
						&answer, 1024, &answerLength, stopAtCr,
						0);
		
		if (err == kKhepNoErr && nlhs >= 1)
			RESULT_OUT = mxCreateString(answer ? answer : "");
	}
#endif
	
	switch (err)
	{
		case kKhepDeviceErr:
			mexErrMsgTxt("KHEPCMD device error");
			break;
		case kKhepComErr:
			mexErrMsgTxt("KHEPCMD read/write error");
			break;
		case kKhepComTimeoutErr:
			mexErrMsgTxt("KHEPCMD timeout error");
			break;
		case kKhepComBadPortErr:
			mexErrMsgTxt("KHEPCMD unsupported serial port");
			break;
		case kKhepComBadBaudRateErr:
			mexErrMsgTxt("KHEPCMD unsupported baud rate");
			break;
	}
}
